; #########################################################################
;
;                             fl2a9
;
;Procedure for converting a floating point value from the FPU st(0) to ASCII
;Limitation: total number of integer and decimal digits is a maximum of 9
;
;Return: EAX  = 0 if error and null string in memory
;        EAX != 0 if success and ASCII string in memory
;        Rounding is done according to the current RC setting of the FPU
;
;Usage:  invoke fl2a9,Dnum,lpBuf
;
;   where:  
;   Dnum  = immediate value or content of a memory variable
;           for the number of decimal digits required in the ASCII string
;   lpBuf = pointer to a memory buffer for the null-terminated ASCII
;           string (a 16-byte buffer is sufficient)
;
;Example: invoke fl2a9, 5, ADDR szFloat1
;
;Note: The top register is also POPed from the FPU in all cases
;      and the float value destroyed
;
;                       by Raymond Filiatreault
;                              May 2004
;                            MASM  syntax
;
; #########################################################################
;
;      .386                   ; minimum processor needed for 32 bit
;      .model flat, stdcall   ; FLAT memory model & STDCALL calling
;      option casemap :none   ; set code to case sensitive
;
; #########################################################################

.code

fl2a9 proc USES edi Dnum:DWORD, lpBuf:DWORD

LOCAL float0      :DWORD
LOCAL bigint      :DWORD

      .if Dnum > 9      ;more than 9 decimal places would exceed the limit

   exiterrorp:
            fstp  float0
   exiterror:
            xor   eax,eax
            mov   edi,lpBuf
            mov   [edi],al
            ret
      .endif

;check if the content of st(0) is a valid floating point value

      fxam
      fstsw ax
      fwait
      sahf              ;transfer result to CPU flag register
      .if   ZERO?       ;would be either 0, empty register or denormalized
            jc exiterrorp ;if empty register
            fstp  float0

      ;A denormalized number would be equivalent to 0 in this context
      ;and considered as valid

   zeroascii:           ;no need for further processing if = 0
            mov   edi,lpBuf
            mov   eax,"0"
            mov   [edi],eax
            ret

      .else
            jpo exiterrorp ;would be either unsupported or a NAN
            jc  exiterrorp ;would be infinity
      .endif
      fst   float0      ;copy to memory as REAL4 to retrieve sign later

      mov   ecx,Dnum
      mov   eax,1

;get 10^(Dnum)

   @@:
      sub   ecx,1
      jc    @F
      imul  eax,10
      jmp   @B

   @@:
      mov   bigint,eax
      fclex             ;clear exception flags
      fimul bigint      ;multiply the float by 10^(Dnum)
      fabs              ;insure it is positive
      fistp bigint      ;retrieve the boosted float as an integer
      fstsw ax          ;to check for exception flags
      fwait
      shr   al,1
      jc    exiterror   ;invalid operation with too large a number
      mov   eax,bigint  ;now in EAX

      .if   eax == 0
            jmp   zeroascii  ;simply display a 0
      .endif

      mov   edi,Dnum    ;use as counter for number of decimal digits
      mov   ecx,10
      pushd 0           ;for later use as terminating 0
   @@:
      xor   edx,edx     ;clear for division
      div   ecx         ;remainder (less than 10) -> EDX (effectively DL)
      add   dl,"0"      ;convert to ascii
      push  edx         ;save each digit on stack
      dec   edi         ;bump down count of decimal digits
      .if   ZERO?
            mov   dl,"."
            push  edx   ;insert decimal point
      .endif
      .if   eax != 0
            jmp   @B    ;continue conversion
      .endif

   @@:
      or    edi,edi     ;would be negative if integer digits already posted
      js    @F
      mov   dl,"0"
      push  edx         ;insert necessary 0s
      dec   edi         ;still used as counter as above
      .if   ZERO?
            mov   dl,"."
            push  edx
      .endif
      jmp   @B

   @@:
      mov   edi,lpBuf
      mov   eax,float0  ;get the copy of the float
      or    eax,eax     ;check for its sign
      jns   @F
      mov   al,"-"
      stosb
   @@:
      pop   eax         ;retrieve each ascii character from the stack
      stosb
      or    al,al
      jnz   @B          ;continue until terminating 0 is retrieved
      or    al,1
      ret

fl2a9 endp

;end
